package com.yunzhi.marketing.broker.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.marketing.base.BaseController;
import com.yunzhi.marketing.base.ResponseBean;
import com.yunzhi.marketing.common.CommConstant;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.marketing.broker.service.IBkNoticeService;
import com.yunzhi.marketing.broker.entity.BkNotice;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * <p>
    * 通知公告 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-07-26
 */

@Api(tags = "通知公告")
@RestController
@RequestMapping("/api")
public class BkNoticeController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(BkNoticeController.class);

    @Autowired
    public IBkNoticeService iBkNoticeService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/{client}/bkNotice",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean bkNoticeList(@ApiParam("客户端") @PathVariable String client,
                                     @ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                     @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                     HttpServletRequest request) throws Exception{

        IPage<BkNotice> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<BkNotice> queryWrapper = new QueryWrapper<>();
        queryWrapper.gt("status", CommConstant.STATUS_DELETE);

        if ("admin".equals(client)) {
            List<String> institutionIds = getInstitutionIds(request);
            queryWrapper.in(null != institutionIds, "institution_id", institutionIds);
        }

        queryWrapper.orderByDesc("create_date");

        IPage<BkNotice> result = iBkNoticeService.page(pg, queryWrapper);
        return ResponseBean.success(result);
    }

//    /**
//     * 保存对象
//     * @param bkNotice 实体对象
//     * @return
//     */
//    @RequestMapping(value="/bkNotice",method= RequestMethod.POST)
//    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
//    public ResponseBean bkNoticeAdd(@ApiParam("保存内容") @RequestBody BkNotice bkNotice) throws Exception{
//
//        if (iBkNoticeService.save(bkNotice)){
//            return ResponseBean.success(bkNotice);
//        }else {
//            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
//    /**
//     * 根据id删除对象
//     * @param id  实体ID
//     */
//    @RequestMapping(value="/bkNotice/{id}", method= RequestMethod.DELETE)
//    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
//    public ResponseBean bkNoticeDelete(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
//        if(iBkNoticeService.removeById(id)){
//            return ResponseBean.success("success");
//        }else {
//            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
//    /**
//     * 修改对象
//     * @param id  实体ID
//     * @param bkNotice 实体对象
//     * @return
//     */
//    @RequestMapping(value="/bkNotice/{id}",method= RequestMethod.PUT)
//    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
//    public ResponseBean bkNoticeUpdate(@ApiParam("对象ID") @PathVariable Integer id,
//                                        @ApiParam("更新内容") @RequestBody BkNotice bkNotice) throws Exception{
//
//        if (iBkNoticeService.updateById(bkNotice)){
//            return ResponseBean.success(iBkNoticeService.getById(id));
//        }else {
//            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/{client}/bkNotice/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean bkNoticeGet(@ApiParam("客户端") @PathVariable String client,
                                    @ApiParam("对象ID") @PathVariable Integer id) throws Exception {
        BkNotice bkNotice = iBkNoticeService.getById(id);
        if (null == bkNotice || CommConstant.STATUS_DELETE.equals(bkNotice.getStatus())) {
            return ResponseBean.error("未找到数据", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        return ResponseBean.success(bkNotice);
    }
}
