package com.yunzhi.marketing.broker.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.marketing.base.BaseController;
import com.yunzhi.marketing.base.ResponseBean;
import com.yunzhi.marketing.broker.service.IBkBankCardService;
import com.yunzhi.marketing.broker.service.IBkInviteRecordService;
import com.yunzhi.marketing.broker.vo.BrokerSummery;
import com.yunzhi.marketing.broker.vo.BrokerVO;
import com.yunzhi.marketing.common.CommConstant;
import com.yunzhi.marketing.common.StringUtils;
import com.yunzhi.marketing.entity.TaPerson;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;


@Api(tags = "全民经纪人(新联康)")
@RestController
@RequestMapping("/api")
public class BrokerController extends BaseController {

    @Autowired
    IBkBankCardService iBkBankCardService;

    @Autowired
    IBkInviteRecordService iBkInviteRecordService;


    @RequestMapping(value="/admin/broker",method= RequestMethod.GET)
    @ApiOperation(value="经纪人列表", notes = "经纪人列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getBrokerList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                      @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                      @ApiParam("手机号") @RequestParam(value ="phone", required = false) String phone,
                                      @ApiParam("姓名") @RequestParam(value ="name", required = false) String name,
                                      @ApiParam("推荐人") @RequestParam(value ="recommendPersonName", required = false) String recommendPersonName,
                                      @ApiParam("排序规则") @RequestParam(value ="sortOrder", defaultValue = "DESC") String sortOrder,
                                      @ApiParam("排序字段") @RequestParam(value ="sortField", defaultValue = "create_date") String sortField) throws Exception {

        IPage<BrokerVO> page = new Page<>(pageNum, pageSize);
        IPage<BrokerVO> result = iTaPersonService.getBrokerPage(page, name, phone, recommendPersonName, sortOrder, sortField);
        return ResponseBean.success(result);
    }


    @RequestMapping(value="/wx/broker",method= RequestMethod.GET)
    @ApiOperation(value="小程序端经纪人列表", notes = "小程序端经纪人列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getWxBrokerList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                      @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                        HttpServletRequest request) throws Exception {
        // 我推荐的经纪人列表
        String personId = getPersonId(request);

        IPage<BrokerSummery> page = new Page<>(pageNum, pageSize);
        IPage<BrokerSummery> result = iTaPersonService.getWxBrokerPage(page, personId);
        return ResponseBean.success(result);
    }

    @RequestMapping(value="/wx/broker/rank",method= RequestMethod.GET)
    @ApiOperation(value="小程序端经纪人榜单", notes = "小程序端经纪人榜单", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getWxBrokerRank(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                        @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                       @ApiParam(value = "榜单类型", allowableValues = "person,money") @RequestParam(value ="rankType") String rankType,
                                        HttpServletRequest request) throws Exception {
        IPage<BrokerSummery> page = new Page<>(pageNum, pageSize);
        IPage<BrokerSummery> result = iTaPersonService.getWxBrokerRankPage(page, rankType);
        return ResponseBean.success(result);
    }


    @RequestMapping(value="/wx/bindBroker",method= RequestMethod.PUT)
    @ApiOperation(value="成为经纪人", notes = "成为经纪人", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean bindBroker(@ApiParam("经纪人信息") @RequestBody BrokerVO brokerVO,
                                   HttpServletRequest request) throws Exception {
        String personId = getPersonId(request);

        brokerVO.setPersonId(personId);
        brokerVO.setPersonType(CommConstant.PERSON_BROKER);

        TaPerson taPerson = new TaPerson();

        BeanUtils.copyProperties(brokerVO, taPerson, "bankCardList");
        taPerson.setTotalCommission(null);
        taPerson.setSettledCommission(null);
        taPerson.setUnsettledCommission(null);


        // 更新数据
        if (!iTaPersonService.updateById(taPerson)) {

            // 如果是有人推荐的, 那么需要生成推荐记录
            if (!StringUtils.isEmpty(brokerVO.getRecommendAgent())) {
                iBkInviteRecordService.createBy(brokerVO.getRecommendAgent(), brokerVO.getPersonId());
            }

            return ResponseBean.error("操作失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }

        return ResponseBean.success(brokerVO);
    }
}
