package com.yunzhi.marketing.redpack.controller;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.marketing.base.BaseController;
import com.yunzhi.marketing.base.ResponseBean;
import com.yunzhi.marketing.redpack.entity.TaOrgRefundApplication;
import com.yunzhi.marketing.redpack.service.ITaOrgRefundApplicationService;

/**
 * <p>
    * 账户退款申请  前端控制器
    * </p>
 *
 * @author fxf
 * @since 2020-06-03
 */
@RestController
@RequestMapping("/")
public class TaOrgRefundApplicationController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaOrgRefundApplicationController.class);

    @Autowired
    public ITaOrgRefundApplicationService iTaOrgRefundApplicationService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/taOrgRefundApplication",method= RequestMethod.GET)
    public ResponseBean taOrgRefundApplicationList(@RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                                   @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize){
        ResponseBean responseBean = new ResponseBean();
        try {
            //使用分页插件
		    IPage<TaOrgRefundApplication> pg = new Page<>(pageNum, pageSize);
            QueryWrapper<TaOrgRefundApplication> queryWrapper = new QueryWrapper<>();
            queryWrapper.orderByDesc("create_date");

            IPage<TaOrgRefundApplication> result = iTaOrgRefundApplicationService.page(pg, queryWrapper);
            responseBean.addSuccess(result);
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taOrgRefundApplicationList -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 保存对象
     * @param taOrgRefundApplication 实体对象
     * @return
     */
    @RequestMapping(value="/taOrgRefundApplication",method= RequestMethod.POST)
    public ResponseBean taOrgRefundApplicationAdd(@RequestBody TaOrgRefundApplication taOrgRefundApplication){
        ResponseBean responseBean = new ResponseBean();
        try {
            if (iTaOrgRefundApplicationService.save(taOrgRefundApplication)){
                responseBean.addSuccess(taOrgRefundApplication);
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taOrgRefundApplicationAdd -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @ResponseBody
    @RequestMapping(value="/taOrgRefundApplication/{id}", method= RequestMethod.DELETE)
    public ResponseBean taOrgRefundApplicationDelete(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            if(iTaOrgRefundApplicationService.removeById(id)){
                responseBean.addSuccess("success");
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taOrgRefundApplicationDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taOrgRefundApplication 实体对象
     * @return
     */
    @RequestMapping(value="/taOrgRefundApplication/{id}",method= RequestMethod.PUT)
    public ResponseBean taOrgRefundApplicationUpdate(@PathVariable Integer id,
                                        @RequestBody TaOrgRefundApplication taOrgRefundApplication){
        ResponseBean responseBean = new ResponseBean();
        try {
            if (iTaOrgRefundApplicationService.updateById(taOrgRefundApplication)){
                responseBean.addSuccess(taOrgRefundApplication);
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taOrgRefundApplicationUpdate -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/taOrgRefundApplication/{id}",method= RequestMethod.GET)
    public ResponseBean taOrgRefundApplicationGet(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            responseBean.addSuccess(iTaOrgRefundApplicationService.getById(id));
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taOrgRefundApplicationDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }
}
