package com.yunzhi.marketing.xlk.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yunzhi.marketing.base.ResponseBean;
import com.yunzhi.marketing.broker.entity.BkAccountRecord;
import com.yunzhi.marketing.broker.entity.BkAgentRule;
import com.yunzhi.marketing.broker.entity.BkInviteRecord;
import com.yunzhi.marketing.broker.mapper.BkAccountRecordMapper;
import com.yunzhi.marketing.broker.mapper.BkAgentRuleMapper;
import com.yunzhi.marketing.broker.mapper.BkInviteRecordMapper;
import com.yunzhi.marketing.center.taUser.entity.TaUser;
import com.yunzhi.marketing.center.taUser.mapper.TaUserMapper;
import com.yunzhi.marketing.common.CommConstant;
import com.yunzhi.marketing.common.StringUtils;
import com.yunzhi.marketing.entity.TaBuilding;
import com.yunzhi.marketing.entity.TaPerson;
import com.yunzhi.marketing.entity.TaRecommendCustomer;
import com.yunzhi.marketing.mapper.TaBuildingMapper;
import com.yunzhi.marketing.mapper.TaPersonMapper;
import com.yunzhi.marketing.mapper.TaRecommendCustomerMapper;
import com.yunzhi.marketing.po.MineRecommendCustomerPO;
import com.yunzhi.marketing.po.TaRecommendCustomerPO;
import com.yunzhi.marketing.xlk.dto.*;
import com.yunzhi.marketing.xlk.entity.*;
import com.yunzhi.marketing.xlk.mapper.*;
import com.yunzhi.marketing.xlk.service.IChannelCustomerService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 渠道报备客户表  服务实现类
 * </p>
 *
 * @author jobob
 * @since 2021-07-19
 */
@Service
public class ChannelCustomerServiceImpl extends ServiceImpl<ChannelCustomerMapper, ChannelCustomer> implements IChannelCustomerService {

    @Autowired
    private ChannelCustomerMapper channelCustomerMapper;

    @Autowired
    private TaRecommendCustomerMapper taRecommendCustomerMapper;

    @Autowired
    private CustomerVisitMapper customerVisitMapper;

    @Autowired
    private CustomerPreparatoryMapper customerPreparatoryMapper;

    @Autowired
    private CustomerSignatoryMapper customerSignatoryMapper;

    @Autowired
    private TaPersonMapper taPersonMapper;

    @Autowired
    private TaBuildingMapper taBuildingMapper;

    @Autowired
    private TaUserMapper taUserMapper;

    @Autowired
    private InstitutionMapper institutionMapper;

    @Autowired
    private BkInviteRecordMapper bkInviteRecordMapper;

    @Autowired
    private BkAgentRuleMapper bkAgentRuleMapper;

    @Autowired
    private BkAccountRecordMapper bkAccountRecordMapper;

    /**
     * 审核
     *
     * @param id
     * @param channelCustomerDTO
     * @return
     */
    @Override
    public ResponseBean auditCustomer(String id, ChannelCustomerDTO channelCustomerDTO) {
        ChannelCustomer customer = channelCustomerMapper.selectById(id);
        if ("fail".equals(channelCustomerDTO.getAuditResult())){
            // 审核不通过
            customer.setStatus("3");
            channelCustomerMapper.updateById(customer);
            return ResponseBean.success("审核成功");
        }

        // 保存到访单
        CustomerVisit customerVisit = channelCustomerDTO.getCustomerVisit();
        customerVisit.setBuildingId(customer.getBuildingId());
        customerVisit.setRecommendPerson(customer.getRecommendPerson());
        TaPerson recommendPerson = taPersonMapper.getById(customer.getRecommendPerson());
        if (null != recommendPerson) {
            customerVisit.setRealtyConsultantName(recommendPerson.getName());
        }
        customerVisit.setRealtyConsultant(channelCustomerDTO.getRealtyConsultant());
        TaPerson channelPerson = taPersonMapper.getById(channelCustomerDTO.getRealtyConsultant());
        if (null != channelPerson) {
            customerVisit.setRealtyConsultantName(channelPerson.getName());
        }
        customerVisit.setOrgId(customer.getOrgId());

        // 审核通过 先查看在ta_recommend_customer 手机号是否存在
        LambdaQueryWrapper<TaRecommendCustomer> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(TaRecommendCustomer::getPhone,customer.getPhone());
        queryWrapper.eq(TaRecommendCustomer::getBuildingId,customer.getBuildingId());
        TaRecommendCustomer recommendCustomer = taRecommendCustomerMapper.selectOne(queryWrapper);
        if (null == recommendCustomer) {
            TaBuilding building = taBuildingMapper.selectById(customer.getBuildingId());
            // 进业务客户表
            TaRecommendCustomer taRecommendCustomer = new TaRecommendCustomer();
            taRecommendCustomer.setName(customer.getName());
            taRecommendCustomer.setPicture(customer.getPicture());
            taRecommendCustomer.setPhone(customer.getPhone());
            taRecommendCustomer.setSex(customer.getSex());
            taRecommendCustomer.setRecommendPerson(customer.getRecommendPerson());
            taRecommendCustomer.setBuildingId(customer.getBuildingId());
            taRecommendCustomer.setIntention(building.getBuildingName());
            taRecommendCustomer.setReportRecommendStatus(CommConstant.RECOMMENDED);
            taRecommendCustomer.setVerifyStatus(CommConstant.VERIFY_AGREE);
            taRecommendCustomer.setCreateDate(LocalDateTime.now());
            taRecommendCustomer.setReportDate(LocalDateTime.now());
            taRecommendCustomer.setOrgId(customer.getOrgId());
            taRecommendCustomer.setPersonId(customer.getPersonId());
            taRecommendCustomer.setEntryType(CommConstant.ENTRY_VERIFY);
            taRecommendCustomer.setStatus(CommConstant.CUSTOMER_VISITE);
            taRecommendCustomer.setRealtyConsultant(channelCustomerDTO.getRealtyConsultant());
            taRecommendCustomer.setCirculationTime(LocalDateTime.now());
            taRecommendCustomer.setInstitutionId(customer.getInstitutionId());
            taRecommendCustomerMapper.insert(taRecommendCustomer);
            // 审核通过
            customer.setStatus("2");
            customer.setCustomerId(taRecommendCustomer.getCustomerId());
            customerVisit.setCustomerId(taRecommendCustomer.getCustomerId());
            customer.setVisitDate(LocalDateTime.now());
            customer.setName(customerVisit.getName());
            channelCustomerMapper.updateById(customer);
        }else if (StringUtils.isEmpty(recommendCustomer.getRealtyConsultant()) &&
                (StringUtils.isEmpty(recommendCustomer.getRecommendPerson()) || recommendCustomer.getRecommendPerson().equals(customer.getRecommendPerson()))){
            // 审核通过
            customer.setStatus("2");
            customer.setCustomerId(recommendCustomer.getCustomerId());
            customerVisit.setCustomerId(recommendCustomer.getCustomerId());
            customer.setVisitDate(LocalDateTime.now());
            customer.setRealtyConsultant(channelCustomerDTO.getRealtyConsultant());
            customer.setName(customerVisit.getName());
            channelCustomerMapper.updateById(customer);
            // 修改状态为确认到访状态
            recommendCustomer.setRecommendPerson(customer.getRecommendPerson());
            recommendCustomer.setVerifyStatus(CommConstant.VERIFY_AGREE);
            recommendCustomer.setStatus(CommConstant.CUSTOMER_VISITE);
            recommendCustomer.setRealtyConsultant(channelCustomerDTO.getRealtyConsultant());
            recommendCustomer.setCirculationTime(LocalDateTime.now());
            taRecommendCustomerMapper.updateById(recommendCustomer);
        }else {
            return ResponseBean.error("此客户是私客或已被报备", ResponseBean.ERROR_UNAVAILABLE);
        }


        customerVisit.setCreateDate(LocalDateTime.now());
        customerVisit.setChannelCustomerId(channelCustomerDTO.getChannelCustomerId());
        customerVisitMapper.insert(customerVisit);

        return ResponseBean.success("审核成功");
    }

    /**
     * 驻场确认到访
     *
     * @param
     * @param params
     * @return
     */
    @Override
    @Transactional
    public ResponseBean markingChannelCustomer(MarkingChannelCustomerDTO params) throws Exception {
        CustomerVisit customerVisit = params.getCustomerVisit();
        // 渠道推荐的客户
        if ("report".equals(params.getType())) {
            ChannelCustomer customer = channelCustomerMapper.selectById(params.getId());

            customerVisit.setBuildingId(customer.getBuildingId());
            customerVisit.setRecommendPerson(customer.getRecommendPerson());
            TaPerson recommendPerson = taPersonMapper.getById(customer.getRecommendPerson());
            if (null != recommendPerson) {
                customerVisit.setRecommendName(recommendPerson.getName());
            }
            customerVisit.setRealtyConsultant(params.getRealtyConsultant());
            TaPerson channelPerson = taPersonMapper.getById(params.getRealtyConsultant());
            if (null != channelPerson) {
                customerVisit.setRealtyConsultantName(channelPerson.getName());
            }
            customerVisit.setOrgId(customer.getOrgId());

            // 审核通过 先查看在ta_recommend_customer 手机号是否存在
            LambdaQueryWrapper<TaRecommendCustomer> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(TaRecommendCustomer::getPhone,customer.getPhone());
            queryWrapper.eq(TaRecommendCustomer::getBuildingId,customer.getBuildingId());
            TaRecommendCustomer recommendCustomer = taRecommendCustomerMapper.selectOne(queryWrapper);
            if (null == recommendCustomer) {
                TaBuilding building = taBuildingMapper.selectById(customer.getBuildingId());
                // 进业务客户表
                TaRecommendCustomer taRecommendCustomer = new TaRecommendCustomer();
                taRecommendCustomer.setName(customerVisit.getName());
                taRecommendCustomer.setPicture(customer.getPicture());
                taRecommendCustomer.setPhone(customer.getPhone());
                taRecommendCustomer.setSex(customer.getSex());
                taRecommendCustomer.setRecommendPerson(customer.getRecommendPerson());
                taRecommendCustomer.setBuildingId(customer.getBuildingId());
                taRecommendCustomer.setIntention(building.getBuildingName());
                taRecommendCustomer.setReportRecommendStatus(CommConstant.RECOMMENDED);
                taRecommendCustomer.setVerifyStatus(CommConstant.VERIFY_AGREE);
                taRecommendCustomer.setCreateDate(LocalDateTime.now());
                taRecommendCustomer.setReportDate(LocalDateTime.now());
                taRecommendCustomer.setOrgId(customer.getOrgId());
                taRecommendCustomer.setPersonId(customer.getPersonId());
                taRecommendCustomer.setEntryType(CommConstant.ENTRY_VERIFY);
                taRecommendCustomer.setStatus(CommConstant.CUSTOMER_VISITE);
                taRecommendCustomer.setRealtyConsultant(params.getRealtyConsultant());
                taRecommendCustomer.setCirculationTime(LocalDateTime.now());
                taRecommendCustomer.setInstitutionId(customer.getInstitutionId());
                taRecommendCustomerMapper.insert(taRecommendCustomer);

                // 审核通过
                customer.setStatus(CommConstant.CHANNEL_CUSTOMER_AGREE);
                customer.setVisitDate(LocalDateTime.now());
                customer.setCustomerId(taRecommendCustomer.getCustomerId());
                customerVisit.setCustomerId(taRecommendCustomer.getCustomerId());
                customer.setName(customerVisit.getName());
                channelCustomerMapper.updateById(customer);
            } else if (StringUtils.isEmpty(recommendCustomer.getRealtyConsultant()) &&
                    (StringUtils.isEmpty(recommendCustomer.getRecommendPerson()) || recommendCustomer.getRecommendPerson().equals(customer.getRecommendPerson()))){
                // 审核通过
                customer.setStatus(CommConstant.CHANNEL_CUSTOMER_AGREE);
                customer.setVisitDate(LocalDateTime.now());
                customer.setCustomerId(recommendCustomer.getCustomerId());
                customerVisit.setCustomerId(recommendCustomer.getCustomerId());
                customer.setName(customerVisit.getName());
                channelCustomerMapper.updateById(customer);

                // 修改状态为确认到访状态
                recommendCustomer.setRecommendPerson(customer.getRecommendPerson());
                recommendCustomer.setVerifyStatus(CommConstant.VERIFY_AGREE);
                recommendCustomer.setStatus(CommConstant.CUSTOMER_VISITE);
                recommendCustomer.setRealtyConsultant(params.getRealtyConsultant());
                recommendCustomer.setCirculationTime(LocalDateTime.now());
                recommendCustomer.setName(customerVisit.getName());
                taRecommendCustomerMapper.updateById(recommendCustomer);
            } else {
                return ResponseBean.error("该客户已到访", ResponseBean.ERROR_UNAVAILABLE);
            }
        }else if ("customer".equals(params.getType())) {
            // 置业顾问 经纪人推荐的客户
            TaRecommendCustomer taRecommendCustomer = taRecommendCustomerMapper.selectById(params.getId());
            taRecommendCustomer.setVerifyStatus(CommConstant.VERIFY_AGREE);
            taRecommendCustomer.setStatus(CommConstant.CUSTOMER_VISITE);
            taRecommendCustomer.setCirculationTime(LocalDateTime.now());
            customerVisit.setCustomerId(taRecommendCustomer.getCustomerId());
            taRecommendCustomerMapper.updateById(taRecommendCustomer);
        } else {
            return ResponseBean.error("请输入正确的类型参数", ResponseBean.ERROR_UNAVAILABLE);
        }

        // 保存到访单
        customerVisit.setCreateDate(LocalDateTime.now());
        LambdaQueryWrapper<CustomerVisit> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(CustomerVisit::getCustomerId,customerVisit.getCustomerId());
        CustomerVisit oldCustomerVisit = customerVisitMapper.selectOne(lambdaQueryWrapper);
        if (null != oldCustomerVisit) {
            throw new Exception("不允许重复报备");
        }
        customerVisitMapper.insert(customerVisit);

        return ResponseBean.success("审核成功");
    }

    /**
     * 驻场 无效报备
     *
     * @param params
     * @return
     */
    @Override
    public ResponseBean invalidCustomerUpdate(MarkingChannelCustomerDTO params) {// 渠道推荐的客户
        if ("report".equals(params.getType())) {
            ChannelCustomer customer = channelCustomerMapper.selectById(params.getId());
            // 审核不通过 先查看在ta_recommend_customer 手机号是否存在
            LambdaQueryWrapper<TaRecommendCustomer> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(TaRecommendCustomer::getPhone,customer.getPhone());
            queryWrapper.eq(TaRecommendCustomer::getBuildingId,customer.getBuildingId());
            TaRecommendCustomer recommendCustomer = taRecommendCustomerMapper.selectOne(queryWrapper);
            if (null != recommendCustomer) {
                // 审核不通过
                recommendCustomer.setVerifyStatus(CommConstant.VERIFY_AGREE);
//                recommendCustomer.setBuildingId(null);
                recommendCustomer.setRealtyConsultant(null);
                recommendCustomer.setRecommendPerson(null);
                taRecommendCustomerMapper.updateById(recommendCustomer);
            }else {
                TaRecommendCustomer taRecommendCustomer = new TaRecommendCustomer();
                taRecommendCustomer.setName(customer.getName());
                taRecommendCustomer.setPicture(customer.getPicture());
                taRecommendCustomer.setPhone(customer.getPhone());
                taRecommendCustomer.setSex(customer.getSex());
                taRecommendCustomer.setRecommendPerson(null);
                taRecommendCustomer.setBuildingId(customer.getBuildingId());
                taRecommendCustomer.setReportRecommendStatus(CommConstant.RECOMMENDED);
                taRecommendCustomer.setVerifyStatus(CommConstant.VERIFY_AGREE);
                taRecommendCustomer.setCreateDate(LocalDateTime.now());
                taRecommendCustomer.setReportDate(LocalDateTime.now());
                taRecommendCustomer.setOrgId(customer.getOrgId());
                taRecommendCustomer.setPersonId(customer.getPersonId());
                taRecommendCustomer.setEntryType(CommConstant.ENTRY_INPUT);
                taRecommendCustomer.setStatus(CommConstant.CUSTOMER_REPORT);
                taRecommendCustomer.setRealtyConsultant(null);
                taRecommendCustomer.setCirculationTime(LocalDateTime.now());
                taRecommendCustomer.setInstitutionId(customer.getInstitutionId());
                taRecommendCustomerMapper.insert(taRecommendCustomer);
                customer.setCustomerId(taRecommendCustomer.getCustomerId());
            }
            customer.setStatus(CommConstant.CHANNEL_CUSTOMER_DISAGREE);
            channelCustomerMapper.updateById(customer);
            return ResponseBean.success("审核成功");
        }
        return ResponseBean.error("无效操作", ResponseBean.ERROR_UNAVAILABLE);
    }

    @Override
    public IPage<ChannelCustomer> getProfileList(IPage<ChannelCustomer> pg,
                                                 Integer orgId,
                                                 String buildingId,
                                                 String name,
                                                 String phone,
                                                 String recommendPersonName,
                                                 String recommendPhone,
                                                 String status, String type, Integer userId,
                                                 List<String> institutions) {
        return channelCustomerMapper.getProfileList(pg, orgId, buildingId, name, phone, recommendPersonName, recommendPhone, status, type, institutions);
    }

    /**
     * 驻场认筹
     *
     * @param params
     * @return
     */
    @Override
    public ResponseBean preparatoryCustomer(MarkingPreparatoryCustomerDTO params) {
        CustomerPreparatory customerPreparatory = params.getCustomerPreparatory();
        TaRecommendCustomer taRecommendCustomer = taRecommendCustomerMapper.selectById(params.getCustomerId());
        taRecommendCustomer.setStatus(CommConstant.CUSTOMER_PREPARATORY);
        taRecommendCustomer.setRealtyConsultant(customerPreparatory.getRealtyConsultant());
        taRecommendCustomerMapper.updateById(taRecommendCustomer);

        LambdaQueryWrapper<ChannelCustomer> channelCustomerLambdaQueryWrapper = new LambdaQueryWrapper<>();
        channelCustomerLambdaQueryWrapper.eq(ChannelCustomer::getCustomerId,params.getCustomerId());
        ChannelCustomer customer = channelCustomerMapper.selectOne(channelCustomerLambdaQueryWrapper);
        if (null != customer) {
            customer.setPreparatoryDate(LocalDateTime.now());
            channelCustomerMapper.updateById(customer);
        }

        customerPreparatory.setCreateDate(LocalDateTime.now());
        customerPreparatory.setBuildingId(taRecommendCustomer.getBuildingId());
        customerPreparatory.setRecommendPerson(taRecommendCustomer.getRecommendPerson());
        TaPerson recommendPerson = taPersonMapper.getById(taRecommendCustomer.getRecommendPerson());
        if (null != recommendPerson) {
            customerPreparatory.setRealtyConsultantName(recommendPerson.getName());
        }
        customerPreparatory.setRealtyConsultant(taRecommendCustomer.getRealtyConsultant());
        TaPerson channelPerson = taPersonMapper.getById(taRecommendCustomer.getRealtyConsultant());
        if (null != channelPerson) {
            customerPreparatory.setRealtyConsultantName(channelPerson.getName());
        }
        customerPreparatory.setOrgId(taRecommendCustomer.getOrgId());
        customerPreparatory.setCustomerId(taRecommendCustomer.getCustomerId());
        if (StringUtils.isEmpty(customerPreparatory.getCustomerPreparatoryId())) {
            customerPreparatoryMapper.insert(customerPreparatory);
        }else {
            customerPreparatoryMapper.updateById(customerPreparatory);
        }

        return ResponseBean.success("审核成功");
    }

    /**
     * 驻场结佣金
     *
     * @param params
     * @return
     */
    @Override
    public ResponseBean commissionCustomer(MarkingCommissionCustomerDTO params) {
        CustomerSignatory customerSignatory = params.getCustomerSignatory();
        TaRecommendCustomer taRecommendCustomer = taRecommendCustomerMapper.selectById(params.getCustomerId());
        taRecommendCustomer.setStatus(CommConstant.CUSTOMER_COMMISSION);
        taRecommendCustomer.setRealtyConsultant(customerSignatory.getRealtyConsultant());
        taRecommendCustomerMapper.updateById(taRecommendCustomer);

        LambdaQueryWrapper<ChannelCustomer> channelCustomerLambdaQueryWrapper = new LambdaQueryWrapper<>();
        channelCustomerLambdaQueryWrapper.eq(ChannelCustomer::getCustomerId,params.getCustomerId());
        ChannelCustomer customer = channelCustomerMapper.selectOne(channelCustomerLambdaQueryWrapper);
        if (null != customer) {
            customer.setCommissionDate(LocalDateTime.now());
            channelCustomerMapper.updateById(customer);
        }

        customerSignatory.setBuildingId(taRecommendCustomer.getBuildingId());
        customerSignatory.setRecommendPerson(taRecommendCustomer.getRecommendPerson());
        TaPerson recommendPerson = taPersonMapper.getById(taRecommendCustomer.getRecommendPerson());
        if (null != recommendPerson) {
            customerSignatory.setRealtyConsultantName(recommendPerson.getName());
        }
        customerSignatory.setRealtyConsultant(taRecommendCustomer.getRealtyConsultant());
        TaPerson channelPerson = taPersonMapper.getById(taRecommendCustomer.getRealtyConsultant());
        if (null != channelPerson) {
            customerSignatory.setRealtyConsultantName(channelPerson.getName());
        }
        customerSignatory.setOrgId(taRecommendCustomer.getOrgId());
        customerSignatory.setCreateDate(LocalDateTime.now());
        customerSignatory.setType("commission");
        customerSignatory.setCustomerId(params.getCustomerId());
        if (StringUtils.isEmpty(customerSignatory.getCustomerSignatoryId())){
            customerSignatoryMapper.insert(customerSignatory);
        }else {
            customerSignatoryMapper.updateById(customerSignatory);
        }

        return ResponseBean.success("审核成功");
    }

    /**
     * 驻场签约
     *
     * @param params
     * @return
     */
    @Override
    public ResponseBean signatoryCustomer(MarkingSignatoryCustomerDTO params) {
        CustomerSignatory customerSignatory = params.getCustomerSignatory();
        TaRecommendCustomer taRecommendCustomer = taRecommendCustomerMapper.selectById(params.getCustomerId());
        taRecommendCustomer.setStatus(CommConstant.CUSTOMER_SIGNED);
        taRecommendCustomer.setRealtyConsultant(customerSignatory.getRealtyConsultant());
        taRecommendCustomerMapper.updateById(taRecommendCustomer);

        LambdaQueryWrapper<ChannelCustomer> channelCustomerLambdaQueryWrapper = new LambdaQueryWrapper<>();
        channelCustomerLambdaQueryWrapper.eq(ChannelCustomer::getCustomerId,params.getCustomerId());
        ChannelCustomer customer = channelCustomerMapper.selectOne(channelCustomerLambdaQueryWrapper);
        if (null != customer) {
            customer.setSignedDate(LocalDateTime.now());
            customer.setRealtyConsultant(customerSignatory.getRealtyConsultant());
            channelCustomerMapper.updateById(customer);
        }

        customerSignatory.setCreateDate(LocalDateTime.now());
        customerSignatory.setBuildingId(taRecommendCustomer.getBuildingId());
        customerSignatory.setRecommendPerson(taRecommendCustomer.getRecommendPerson());
        TaPerson recommendPerson = taPersonMapper.getById(taRecommendCustomer.getRecommendPerson());
        if (null != recommendPerson) {
            customerSignatory.setRealtyConsultantName(recommendPerson.getName());
        }
        customerSignatory.setRealtyConsultant(taRecommendCustomer.getRealtyConsultant());
        TaPerson channelPerson = taPersonMapper.getById(taRecommendCustomer.getRealtyConsultant());
        if (null != channelPerson) {
            customerSignatory.setRealtyConsultantName(channelPerson.getName());
        }
        customerSignatory.setOrgId(taRecommendCustomer.getOrgId());
        customerSignatory.setType("signatory");
        customerSignatory.setCustomerId(params.getCustomerId());
        if (StringUtils.isEmpty(customerSignatory.getCustomerSignatoryId())) {
            customerSignatoryMapper.insert(customerSignatory);
        }else {
            customerSignatoryMapper.updateById(customerSignatory);
        }

        return ResponseBean.success("审核成功");
    }

    /**
     * 查询经纪人的我的客户列表
     *
     * @param pageNumber
     * @param pageSize
     * @param recommendCustomerDTO
     * @return
     */
    @Override
    public ResponseBean getCustomersIRecommended(int pageNumber, int pageSize, RecommendCustomerDTO recommendCustomerDTO) {
        IPage<MineRecommendCustomerPO> page = new Page<MineRecommendCustomerPO>(pageNumber, pageSize);
        IPage<MineRecommendCustomerPO> customersIRecommended = channelCustomerMapper.getCustomersIRecommended(page, recommendCustomerDTO);
        return ResponseBean.success(customersIRecommended);
    }

    @Override
    public ChannelCustomer getMoreInfoById(String id) {
        return channelCustomerMapper.getMoreInfoById(id);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateBizNode(MarkingSignatoryCustomerDTO params, Integer bizStatus, String bizType) throws Exception {
        LocalDateTime now = LocalDateTime.now();
        CustomerSignatory customerSignatory = params.getCustomerSignatory();
        customerSignatory.setCreateDate(now);
        customerSignatory.setType(bizType);
        customerSignatory.setCustomerId(params.getCustomerId());

        // 以最新数据为准, 更新客户信息
        TaRecommendCustomer taRecommendCustomer = taRecommendCustomerMapper.selectById(customerSignatory.getCustomerId());
        if (!StringUtils.isEmpty(customerSignatory.getRealtyConsultant())) {
            taRecommendCustomer.setRealtyConsultant(customerSignatory.getRealtyConsultant());
        } else {
            customerSignatory.setRealtyConsultant(taRecommendCustomer.getRealtyConsultant());
        }
        taRecommendCustomer.setStatus(bizStatus);
        taRecommendCustomerMapper.updateById(taRecommendCustomer);
        //
        customerSignatory.setOrgId(taRecommendCustomer.getOrgId());
        customerSignatory.setBuildingId(taRecommendCustomer.getBuildingId());
        TaBuilding taBuilding = taBuildingMapper.selectById(taRecommendCustomer.getBuildingId());
        if (taBuilding == null) {
            throw new Exception("校验客户及楼盘信息异常");
        }

        // 推荐人
        customerSignatory.setRecommendPerson(taRecommendCustomer.getRecommendPerson());
        TaPerson recommendPerson = taPersonMapper.getById(taRecommendCustomer.getRecommendPerson());
        if (null != recommendPerson) {
            customerSignatory.setRecommendName(recommendPerson.getName());
        }

        // 更新报备记录
        LambdaQueryWrapper<ChannelCustomer> channelCustomerLambdaQueryWrapper = new LambdaQueryWrapper<>();
        channelCustomerLambdaQueryWrapper.eq(ChannelCustomer::getCustomerId,params.getCustomerId());
        ChannelCustomer customer = channelCustomerMapper.selectOne(channelCustomerLambdaQueryWrapper);
        if (null != customer) {
            if (CommConstant.CUSTOMER_SIGNED.equals(bizStatus)) {
                customer.setSignedDate(now);
            } else if (CommConstant.CUSTOMER_COMMISSION.equals(bizStatus)) {
                customer.setCommissionDate(now);
            }

            if (StringUtils.isEmpty(customer.getRealtyConsultant())) {
                customer.setRealtyConsultant(customerSignatory.getRealtyConsultant());
            }

            // 全民经纪人佣金处理
            boolean isBroker = CommConstant.PERSON_BROKER.equals(customer.getRecommendPersonType());
            if (isBroker) {
                // 处理推荐金额问题
                processInviteRecord(recommendPerson, taBuilding);
                // 记录佣金金额
                if (null != params.getTotalCommission() && params.getTotalCommission() > 0) {
                    processCommission(params, customer, recommendPerson, bizType, taBuilding);
                }
            }

            channelCustomerMapper.updateById(customer);
        }

        // 置业顾问-接待人
        if (!StringUtils.isEmpty(customerSignatory.getRealtyConsultant())) {
            TaPerson realtyConsultant = taPersonMapper.getById(customerSignatory.getRealtyConsultant());
            if (null != realtyConsultant) {
                customerSignatory.setRealtyConsultantName(realtyConsultant.getName());
            }
        }

        // 更新
        if (StringUtils.isEmpty(customerSignatory.getCustomerSignatoryId())){
            customerSignatoryMapper.insert(customerSignatory);
        } else {
            customerSignatoryMapper.updateById(customerSignatory);
        }
    }

    private Integer intIfNvl(Integer i) {
        return i == null ? 0 : i;
    }

    private void processCommission(MarkingSignatoryCustomerDTO params, ChannelCustomer customer, TaPerson recommendPerson, String bizType, TaBuilding taBuilding) {
        if (null == recommendPerson) return;

        // 为了防止数据有改动, 比如更新操作
        // 那么需要把原来的值减掉,再使用新的值
        Integer totalCommission = intIfNvl(recommendPerson.getTotalCommission()) -
                intIfNvl(customer.getTotalCommission()) +
                intIfNvl(params.getTotalCommission());
        recommendPerson.setTotalCommission(totalCommission);
        customer.setTotalCommission(intIfNvl(params.getTotalCommission()));

        Integer settledCommission = intIfNvl(bkAccountRecordMapper.getTotalChargesBy(customer.getChannelCustomerId(), bizType));
        settledCommission += intIfNvl(params.getSettledCommission()); // 最新的需要加上本次结算佣金
        recommendPerson.setSettledCommission(intIfNvl(recommendPerson.getSettledCommission()) -
                intIfNvl(customer.getSettledCommission()) +
                settledCommission);
        customer.setSettledCommission(settledCommission);
        customer.setUnsettledCommission(customer.getTotalCommission() - customer.getSettledCommission());

        // 更新记录, customer 在本函数外部更新
        taPersonMapper.updateById(recommendPerson);

        // 生成佣金记录
        BkAccountRecord bkAccountRecord = bkAccountRecordMapper.getRecordBy(customer.getChannelCustomerId(), bizType);
        if (bkAccountRecord != null) {
            bkAccountRecord.setCharges(params.getSettledCommission());
            bkAccountRecordMapper.updateById(bkAccountRecord);
        } else {
            bkAccountRecord = new BkAccountRecord();
            bkAccountRecord.setChannelCustomerId(customer.getChannelCustomerId());
            bkAccountRecord.setPersonId(recommendPerson.getPersonId());
            bkAccountRecord.setBuildingId(taBuilding.getBuildingId());
            bkAccountRecord.setBuildingName(StringUtils.ifNull(taBuilding.getName(), taBuilding.getBuildingName()));
            bkAccountRecord.setCustomerName(customer.getName());
            bkAccountRecord.setCharges(intIfNvl(params.getSettledCommission()));
            bkAccountRecord.setChargeType(1);
            bkAccountRecord.setChargeCode(bizType);
            bkAccountRecord.setStatus(CommConstant.STATUS_NORMAL);
            bkAccountRecord.setCreateDate(LocalDateTime.now());
            bkAccountRecordMapper.insert(bkAccountRecord);
        }
    }

    /**
     * 处理推荐金
     * @param recommendPerson
     * @param taBuilding
     */
    private void processInviteRecord(TaPerson recommendPerson, TaBuilding taBuilding) {
        // 推荐人员
        String personId = recommendPerson.getRecommendAgent();
        if (StringUtils.isEmpty(personId)) return;

        // 查询推荐记录
        BkInviteRecord bkInviteRecord = bkInviteRecordMapper.getByPerson(personId, recommendPerson.getPersonId());
        if (null == bkInviteRecord) {
            bkInviteRecordMapper.createBy(personId, recommendPerson.getPersonId());
            bkInviteRecord = bkInviteRecordMapper.getByPerson(personId, recommendPerson.getPersonId());
        }

        // 大于0 代表已经计算过推荐费用
        if (bkInviteRecord.getReferralFee() > 0) return;

        // 查询推荐费用
        BkAgentRule agentRule = bkAgentRuleMapper.getByCity(taBuilding.getCityId());
        if (null == agentRule) return;

        bkInviteRecord.setReferralFee(agentRule.getReferralFee());
        bkInviteRecordMapper.updateById(bkInviteRecord);
    }
}
